@@ -17,6 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#define _GNU_SOURCE
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "mbmenu.h"
 
 #define MBMAX(x,y) ((x>y)?(x):(y))
@@ -294,7 +300,9 @@ mb_menu_set_trans(MBMenu *mbmenu, int trans)
 }
 
 void
-mb_menu_set_col(MBMenu *mbmenu, MBMenuColorElement element, char *col_spec)
+mb_menu_set_col(MBMenu            *mbmenu, 
+		MBMenuColorElement element, 
+		char              *col_spec)
 {
   
   switch (element)
@@ -662,23 +670,6 @@ mb_menu_check_scroll_button(MBMenu *mb,MBMenuMenu *m, int y_pos)
       return WANT_SCROLL_DOWN;
     }
 
-  /*
-  for(tmpi = m->too_big_start_item;
-      tmpi != NULL; 
-      tmpi = tmpi->next_item)
-    if ( (tmpi->y + tmpi->h) >= (m->height - SCROLL_BUTT_H) )
-      {
-	h = tmpi->y;
-	break;
-      }
-  
-  if (y_pos > h)
-    {
-      MENUDBG("%s() retruning want scroll up\n", __func__);
-      return WANT_SCROLL_UP;
-    }
-  */
-
   if (m->too_big_end_item
       && y_pos > (m->too_big_end_item->y+m->too_big_end_item->h))
     return WANT_SCROLL_UP;
@@ -724,25 +715,25 @@ menu_set_theme_from_root_prop(MBMenu *mb)
 	      if (mb_dotdesktop_get(theme, "MenuBgColor"))
 		{
 		  mb_menu_set_col(mb, MBMENU_SET_BG_COL, 
-				  mb_dotdesktop_get(theme, "MenuBgColor"));
+				  (char*)mb_dotdesktop_get(theme, "MenuBgColor"));
 		}
 
 	      if (mb_dotdesktop_get(theme, "MenuFont"))
 		{
 		  mb_menu_set_font (mb, 
-				    mb_dotdesktop_get(theme, "MenuFont"));
+				    (char*)mb_dotdesktop_get(theme, "MenuFont"));
 		}
 
 	      if (mb_dotdesktop_get(theme, "MenuFgColor"))
 		{
 		  mb_menu_set_col(mb, MBMENU_SET_FG_COL, 
-				  mb_dotdesktop_get(theme, "MenuFgColor"));
+				  (char*)mb_dotdesktop_get(theme, "MenuFgColor"));
 		}
 
 	      if (mb_dotdesktop_get(theme, "MenuHlColor"))
 		{
 		  mb_menu_set_col(mb, MBMENU_SET_HL_COL, 
-				  mb_dotdesktop_get(theme, "MenuHlColor"));
+				  (char*)mb_dotdesktop_get(theme, "MenuHlColor"));
 		    mb->have_highlight_col = True;
 		}
 	      else mb->have_highlight_col = False;
@@ -750,7 +741,7 @@ menu_set_theme_from_root_prop(MBMenu *mb)
 	      if (mb_dotdesktop_get(theme, "MenuBdColor"))
 		{
 		  mb_menu_set_col(mb, MBMENU_SET_BD_COL, 
-				  mb_dotdesktop_get(theme, "MenuBdColor"));
+				  (char*)mb_dotdesktop_get(theme, "MenuBdColor"));
 		}
 
 	      /* xxx currently broke xxx
