@@ -18,6 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#define _GNU_SOURCE
+
 #include <strings.h>
 #include "mbpixbuf.h"
 
@@ -41,28 +43,37 @@
     (composite) = (ush)((temp + (temp >> 8)) >> 8);             } \
 }
 
+#ifdef WORDS_BIGENDIAN
+#define  SHORT_FROM_2BYTES(p) ( *(p+1) | (*(p) << 8) )
+#define  2BYTES_FROM_SHORT(p,s)                       \
+     *((p)+1)   = (unsigned char) s;                  \
+     *(p)       = (unsigned char) ((s >> 8) & 0xff);    
+#else
+#define  SHORT_FROM_2BYTES(p) ( *(p) | (*((p)+1) << 8) )
+#define  BYTES_FROM_SHORT(p,s)                       \
+     *(p)   = (unsigned char) s;                      \
+     *((p)+1) = (unsigned char) ((s >> 8) & 0xff);    
+#endif
 
-#define internal_16bpp_pixel_next(p) \
-      (p) += 2
-
-#define internal_16bpp_pixel_to_rgb(p,r,g,b)          \
-      {                                               \
-         unsigned short s = ( *p | (*(p+1) << 8));    \
-         (r) = (( s & 0xf800) >> 8);                  \
-         (g) = (( s & 0x07e0) >> 3);                  \
-         (b) = (( s & 0x001f) << 3);                  \
+#define internal_16bpp_pixel_to_rgb(p,r,g,b)           \
+      {                                                \
+         unsigned short s = SHORT_FROM_2BYTES(p);      \
+         (r) = (( s & 0xf800) >> 8);                   \
+         (g) = (( s & 0x07e0) >> 3);                   \
+         (b) = (( s & 0x001f) << 3);                   \
       }
 
 
-#define internal_rgb_to_16bpp_pixel(r,g,b,p)         \
+#define internal_rgb_to_16bpp_pixel(r,g,b,p)          \
      {                                                \
       unsigned short s = (  (((b) >> 3) & 0x001f) |   \
                             (((g) << 3) & 0x07e0) |   \
                             (((r) << 8) & 0xf800) );  \
-     *(p)   = (unsigned char) s;                      \
-     *((p)+1) = (unsigned char) ((s >> 8) & 0xff);    \
+      BYTES_FROM_SHORT(p,s)                           \
      }
 
+#define internal_16bpp_pixel_next(p) \
+      (p) += 2
 
 #define IN_REGION(x,y,w,h) ( (x) > -1 && (x) < (w) && (y) > -1 && (y) <(h) ) 
 
@@ -320,7 +331,7 @@ _load_xpm_file( MBPixbuf *pb, const char *filename, int *w, int *h, int *has_alp
   int pc, c = ' ', i = 0, j = 0, k = 0, ncolors = 0, cpp = 0, 
     comment = 0, transp = 0, quote = 0, context = 0,  len, done = 0;
 
-  char *line, s[256], tok[128], col[256];
+  char *line, s[257], tok[128], col[256];
 
   XColor xcol;
   int lsz = 256;
@@ -407,7 +418,7 @@ _load_xpm_file( MBPixbuf *pb, const char *filename, int *w, int *h, int *has_alp
 		      col[0] = 0;
 		      s[0] = 0;
 		      len = strlen(line);
-		      strncpy(cmap[j].str, line, cpp);
+		      strncpy((char*)cmap[j].str, line, cpp);
 		      cmap[j].str[cpp] = 0;
 		      cmap[j].r = -1;
 		      cmap[j].transp = 0;
@@ -526,7 +537,7 @@ _load_xpm_file( MBPixbuf *pb, const char *filename, int *w, int *h, int *has_alp
 			  i--;
 			  for (j = 0; j < ncolors; j++)
 			    {
-			      if (!strcmp(col, cmap[j].str))
+			      if (!strcmp(col, (char*)cmap[j].str))
 				{
 				  if (transp && cmap[j].transp)
 				    {
@@ -694,6 +705,7 @@ _paletteAlloc(MBPixbuf *pb)
   return colnum;
 }
 
+
 static unsigned long
 mb_pixbuf_get_pixel(MBPixbuf *pb, int r, int g, int b, int a)
 {
@@ -777,6 +789,13 @@ mb_pixbuf_get_pixel(MBPixbuf *pb, int r, int g, int b, int a)
   return 0;
 }
 
+unsigned long
+mb_pixbuf_lookup_x_pixel(MBPixbuf *pb, int r, int g, int b, int a)
+{
+  return mb_pixbuf_get_pixel(pb, r, g, b, a);
+}
+
+
 MBPixbuf *
 mb_pixbuf_new(Display *dpy, int scr)
 {
@@ -888,6 +907,7 @@ mb_pixbuf_new_extended(Display *dpy,
 	  fprintf(stderr, "mbpixbuf: unable to use XShm. DISPLAY remote?\n");
 	  pb->have_shm = False;
 	}
+      else XShmDetach(pb->dpy, &shminfo);
 
       shmdt(shminfo.shmaddr);
       shmctl(shminfo.shmid, IPC_RMID, 0);
@@ -1067,7 +1087,8 @@ mb_pixbuf_img_new_from_x_drawable (MBPixbuf *pb,
   int num_of_cols = 1 << pb->depth;
 
   Window chld;
-  unsigned int rx, rw, rh, rb, rdepth;
+  int          rx;
+  unsigned int rw, rh, rb, rdepth;
 
   XShmSegmentInfo shminfo; 
 
@@ -2013,7 +2034,7 @@ mb_pixbuf_img_get_pixel (MBPixbuf      *pixbuf,
 
   if (pixbuf->internal_bytespp == 2)
     {
-      int offset = (((y)*img->width*idx)+((x)*idx));
+      int offset = ( (y * img->width * idx) + ( x * idx ) );
       internal_16bpp_pixel_to_rgb(img->rgba+offset, *r, *g, *b);
 
       if (img->has_alpha)
