@@ -26,6 +26,14 @@
 
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include "mbtray.h"
 
 #include <strings.h>
@@ -202,7 +210,7 @@ _get_server_time(Display* dpy)
 {
   XEvent xevent;
   Atom timestamp_atom = XInternAtom(dpy, "_MB_DOCK_TIMESTAMP", False);
-  char c = 'a';
+  unsigned char c = 'a';
 
   XChangeProperty (dpy, RootWindow(dpy, DefaultScreen(dpy)), 
 		   timestamp_atom, timestamp_atom,
@@ -254,8 +262,7 @@ _xsettings_notify_cb (const char       *name,
 static Bool
 get_xevent_timed( MBTrayApp* mb, XEvent* event_return )
 {
-
-  struct timeval tv_tmp;
+  static struct timeval tv_tmp;
 
   if (mb->poll_fd == -1 && mb->poll_timeout == NULL) 
     {
@@ -274,8 +281,17 @@ get_xevent_timed( MBTrayApp* mb, XEvent* event_return )
       if (mb->poll_fd != -1)
 	FD_SET(mb->poll_fd, &readset);
 
-      if (mb->poll_timeout) /* select resets, so make copy  */
-	memcpy(&tv_tmp, mb->poll_timeout, sizeof(struct timeval));
+      if (mb->poll_timeout)
+	{
+	  if (tv_tmp.tv_sec <= 0 && tv_tmp.tv_usec <= 0)
+	    {
+	      /* select sets tv_tmp with time left so we let it decrement
+	       * to 0 and then reset.	  
+               * XXX According to man page Im not sure how portable this is.
+	      */
+	      memcpy(&tv_tmp, mb->poll_timeout, sizeof(struct timeval));
+	    }
+	}
 
       if (select( (fd > mb->poll_fd) ? fd+1 : mb->poll_fd+1 , 
 		  &readset, NULL, NULL, 
@@ -310,6 +326,12 @@ mb_tray_app_set_poll_timeout ( MBTrayApp *mb,
 {
   if (mb->poll_timeout) free (mb->poll_timeout);
 
+  if (tv == NULL)
+    {
+      mb->poll_timeout = NULL;
+      return;
+    }
+
   mb->poll_timeout = malloc(sizeof(struct timeval));
   memcpy(mb->poll_timeout, tv, sizeof(struct timeval));
 
@@ -423,20 +445,12 @@ _set_icon_hint(MBTrayApp *mb, MBPixbuf *pb, MBPixbufImage *img)
       data[0] = img->width;
       data[1] = img->height;
 
-      /*
-      for( i=2; i < (img->width*img->height)+2; i++)
-	{
-	  idx = (i-2)*4;
-	  data[i] = img->rgba[idx] << 16 | img->rgba[idx+1] << 8  
-	    | img->rgba[idx+2] | img->rgba[idx+3] << 24;  
-	}
-      */
 
       for (y=0; y<img->height; y++)
 	for (x=0; x<img->width; x++)
 	  {
 	    mb_pixbuf_img_get_pixel (pb, img, x, y, &r, &g, &b, &a);
-	    data[i] = ( (a << 24)|(r << 16)|(g << 8)|b );
+	    data[i] = ( (a << 24) | (r << 16)| (g << 8) | b );
 	    i++;
 	  }
       
@@ -613,7 +627,7 @@ mb_tray_app_new_with_display ( unsigned char         *app_name,
 
   mb->tray_id = 0;
 
-  mb->app_name = app_name ? strdup(app_name) : strdup("unnamed");
+  mb->app_name = (unsigned char*)(app_name ? strdup((char*)app_name) : strdup("unnamed"));
 
   mb->have_cached_bg = False; 
   mb->cached_bg      = NULL;
@@ -636,7 +650,7 @@ mb_tray_app_set_name (MBTrayApp     *mb,
 		      unsigned char *name)
 {
   if (mb->app_name) free(mb->app_name);
-  mb->app_name = strdup(name);
+  mb->app_name = (unsigned char*)strdup((char*)name);
 }
 
 void
@@ -647,7 +661,7 @@ mb_tray_app_set_context_info (MBTrayApp     *mb,
 
   if (mb->context_info) free(mb->context_info);
 
-  mb->context_info = strdup(info);
+  mb->context_info = (unsigned char*)strdup((char*)info);
 
   if (mb->win) _set_win_context_hint(mb);
 
@@ -1053,7 +1067,7 @@ _set_win_utf8_name(MBTrayApp *mb)
 		   mb->atoms[ATOM_NET_WM_NAME], 
 		   mb->atoms[ATOM_UTF8_STRING], 
 		   8, 
-		   PropModeReplace, mb->app_name, strlen(mb->app_name));
+		   PropModeReplace, mb->app_name, strlen((char*)mb->app_name));
 }
 
 static void
@@ -1066,7 +1080,7 @@ _set_win_context_hint(MBTrayApp *mb)
 		       mb->atoms[ATOM_UTF8_STRING], 
 		       8, 
 		       PropModeReplace, 
-		       mb->context_info, strlen(mb->context_info));
+		       mb->context_info, strlen((char*)mb->context_info));
     }
 }
 
@@ -1168,7 +1182,7 @@ _init_docking (MBTrayApp *mb )
   
   TRAYDBG("%s() set w: %i, h: %i\n", __func__, mb->w, mb->h);
 
-  XSetStandardProperties(mb->dpy, mb->win, mb->app_name,
+  XSetStandardProperties(mb->dpy, mb->win, (char*)mb->app_name,
 			 NULL, 0, NULL, 0, &size_hints);
 
   _set_win_utf8_name(mb);
@@ -1184,7 +1198,7 @@ void
 mb_tray_app_tray_send_message(MBTrayApp *mb, unsigned char* msg, int timeout)
 {
    unsigned char buf[20];
-   int msg_len = strlen(msg);
+   int msg_len = strlen((char*)msg);
    int id = 12345; /* TODO id should unique */
    int bytes_sent = 0;
    
